@@ -6,18 +6,32 @@ import threading
 import warnings
 import collections
 import statsd
+import re
 
-from django.conf import settings
+from django.utils import deprecation
 from django.core import exceptions
 
-from django_statsd import utils
+from . import utils
+from . import settings
 
 logger = logging.getLogger(__name__)
 
+
+TAGS_LIKE_SUPPORTED = ['=', '_is_']
 try:
-    TRACK_MIDDLEWARE = getattr(settings, 'STATSD_TRACK_MIDDLEWARE', False)
+    MAKE_TAGS_LIKE = settings.STATSD_TAGS_LIKE
+    if MAKE_TAGS_LIKE is not None:
+        if MAKE_TAGS_LIKE is True:
+            MAKE_TAGS_LIKE = '_is_'
+        elif MAKE_TAGS_LIKE not in TAGS_LIKE_SUPPORTED:
+            MAKE_TAGS_LIKE = False
+            warnings.warn(
+                'Unsupported `STATSD_TAGS_LIKE` setting. '
+                'Please, choose from %r' % TAGS_LIKE_SUPPORTED
+            )
+
 except exceptions.ImproperlyConfigured:
-    TRACK_MIDDLEWARE = False
+    MAKE_TAGS_LIKE = False
 
 
 class WithTimer(object):
@@ -42,9 +56,8 @@ class Client(object):
     class_ = statsd.Client
 
     def __init__(self, prefix='view'):
-        global_prefix = getattr(settings, 'STATSD_PREFIX', None)
-        if global_prefix:
-            prefix = '%s.%s' % (global_prefix, prefix)
+        if settings.STATSD_PREFIX:
+            prefix = '%s.%s' % (settings.STATSD_PREFIX, prefix)
         self.prefix = prefix
         self.data = collections.defaultdict(int)
 
@@ -102,7 +115,7 @@ class Timer(Client):
         for k in list(self.data.keys()):
             client.send(k, self.data.pop(k))
 
-        if settings.DEBUG:
+        if settings.STATSD_DEBUG:
             assert not self.starts, ('Timer(s) %r were started but never '
                                      'stopped' % self.starts)
 
@@ -110,19 +123,31 @@ class Timer(Client):
         return WithTimer(self, key)
 
 
-class StatsdMiddleware(object):
+class StatsdMiddleware(deprecation.MiddlewareMixin):
     scope = threading.local()
 
-    def __init__(self):
+    def __init__(self, get_response=None):
+        deprecation.MiddlewareMixin.__init__(self, get_response)
         self.scope.timings = None
         self.scope.counter = None
 
+    @classmethod
+    def skip_view(cls, view_name):
+        for pattern in settings.STATSD_VIEWS_TO_SKIP:
+            if re.match(pattern, view_name):
+                logger.debug("Skipping metric `{}`".format(view_name))
+                return True
+
+        return False
+
     @classmethod
     def start(cls, prefix='view'):
         cls.scope.timings = Timer(prefix)
         cls.scope.timings.start('total')
         cls.scope.counter = Counter(prefix)
         cls.scope.counter.increment('hit')
+        cls.scope.counter_codes = Counter(prefix)
+        cls.scope.counter_codes.increment('hit')
         cls.scope.counter_site = Counter(prefix)
         cls.scope.counter_site.increment('hit')
         return cls.scope
@@ -138,12 +163,12 @@ class StatsdMiddleware(object):
     def process_request(self, request):
         # store the timings in the request so it can be used everywhere
         request.statsd = self.start()
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             self.scope.timings.start('process_request')
         self.view_name = None
 
     def process_view(self, request, view_func, view_args, view_kwargs):
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.start('process_view')
 
         # View name is defined as module.view
@@ -157,23 +182,46 @@ class StatsdMiddleware(object):
             self.view_name = '%s.%s' % (
                 self.view_name, view_func.__class__.__name__)
 
+        if MAKE_TAGS_LIKE:
+            self.view_name = self.view_name.replace('.', '_')
+            self.view_name = 'view' + MAKE_TAGS_LIKE + self.view_name
+
     def process_response(self, request, response):
-        if TRACK_MIDDLEWARE:
+        if self.view_name and self.__class__.skip_view(self.view_name):
+            return response
+
+        self.scope.counter_codes.increment(
+            str(response.status_code // 100) + 'xx')
+        self.scope.counter_codes.submit('http_codes')
+
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.stop('process_response')
-        method = request.method.lower()
-        if request.is_ajax():
-            method += '_ajax'
-        if getattr(self, 'view_name', None):
-            self.stop(method, self.view_name)
+        if MAKE_TAGS_LIKE:
+            method = 'method' + MAKE_TAGS_LIKE
+            method += request.method.lower().replace('.', '_')
+
+            is_ajax = 'is_ajax' + MAKE_TAGS_LIKE
+            is_ajax += str(request.is_ajax()).lower()
+
+            if getattr(self, 'view_name', None):
+                self.stop(method, self.view_name, is_ajax)
+        else:
+            method = request.method.lower()
+            if request.is_ajax():
+                method += '_ajax'
+            if getattr(self, 'view_name', None):
+                self.stop(method, self.view_name)
         self.cleanup(request)
         return response
 
     def process_exception(self, request, exception):
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.stop('process_exception')
+        self.scope.counter_codes.increment('5xx')
+        self.scope.counter_codes.submit('http_codes')
 
     def process_template_response(self, request, response):
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.stop('process_template_response')
         return response
 
@@ -184,44 +232,31 @@ class StatsdMiddleware(object):
         request.statsd = None
 
 
-class StatsdMiddlewareTimer(object):
+class StatsdMiddlewareTimer(deprecation.MiddlewareMixin):
 
     def process_request(self, request):
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.stop('process_request')
 
     def process_view(self, request, view_func, view_args, view_kwargs):
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.stop('process_view')
 
     def process_response(self, request, response):
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.start('process_response')
         return response
 
     def process_exception(self, request, exception):
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.start('process_exception')
 
     def process_template_response(self, request, response):
-        if TRACK_MIDDLEWARE:
+        if settings.STATSD_TRACK_MIDDLEWARE:
             StatsdMiddleware.scope.timings.start('process_template_response')
         return response
 
 
-class TimingMiddleware(StatsdMiddleware):
-
-    @classmethod
-    def deprecated(cls, *args, **kwargs):
-        warnings.warn(
-            'The `TimingMiddleware` has been deprecated in favour of '
-            'the `StatsdMiddleware`. Please update your middleware settings',
-            DeprecationWarning
-        )
-
-    __init__ = deprecated
-
-
 class DummyWith(object):
 
     def __enter__(self):
