@@ -5,7 +5,8 @@ try:
     from coffin.template import loader
     if not hasattr(loader, 'statsd_patched'):
         loader.statsd_patched = True
-        loader.render_to_string = django_statsd.named_wrapper('render_jinja', loader.render_to_string)
+        loader.render_to_string = django_statsd.named_wrapper(
+            'render_jinja', loader.render_to_string)
 
 except ImportError:
     pass
@@ -14,8 +15,8 @@ try:
     from django.template import loader
     if not hasattr(loader, 'statsd_patched'):
         loader.statsd_patched = True
-        loader.render_to_string = django_statsd.named_wrapper('render_django', loader.render_to_string)
+        loader.render_to_string = django_statsd.named_wrapper(
+            'render_django', loader.render_to_string)
 
 except ImportError:
     pass
-
