@@ -1,18 +1,21 @@
 import statsd
-from django.conf import settings
+from . import settings
 
 
-def get_connection(host=None, port=None, sample_rate=None):
+def get_connection(host=None, port=None, sample_rate=None, disabled=None):
     if not host:
-        host = getattr(settings, 'STATSD_HOST', '127.0.0.1')
+        host = settings.STATSD_HOST
 
     if not port:
-        port = getattr(settings, 'STATSD_PORT', 8125)
+        port = settings.STATSD_PORT
 
     if not sample_rate:
-        sample_rate = getattr(settings, 'STATSD_SAMPLE_RATE', 1.0)
+        sample_rate = settings.STATSD_SAMPLE_RATE
 
-    return statsd.Connection(host, port, sample_rate)
+    if not disabled:
+        disabled = settings.STATSD_DISABLED
+
+    return statsd.Connection(host, port, sample_rate, disabled)
 
 
 def get_client(name, connection=None, class_=statsd.Client):
